@@ -28,8 +28,9 @@ from gi.repository import Gdk, Gio, GLib, GObject, Gtk, Pango
 
 from meld import tree
 from meld.conf import _
+from meld.externalhelpers import open_files_external
 from meld.iohelpers import trash_or_confirm
-from meld.melddoc import MeldDoc, open_files_external
+from meld.melddoc import MeldDoc
 from meld.misc import error_dialog, read_pipe_iter
 from meld.recent import RecentType
 from meld.settings import bind_settings, settings
@@ -115,7 +116,7 @@ class VcTreeStore(tree.DiffTreeStore):
 
 
 @Gtk.Template(resource_path='/org/gnome/meld/ui/vcview.ui')
-class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
+class VcView(Gtk.Box, tree.TreeviewCommon, MeldDoc):
 
     __gtype_name__ = "VcView"
 
@@ -153,6 +154,15 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         'ignored': ('vc-status-ignored', Entry.is_ignored),
     }
 
+    replaced_entries = (
+        # Remove Ctrl+Page Up/Down bindings. These are used to do horizontal
+        # scrolling in GTK by default, but we preference easy tab switching.
+        (Gdk.KEY_Page_Up, Gdk.ModifierType.CONTROL_MASK),
+        (Gdk.KEY_KP_Page_Up, Gdk.ModifierType.CONTROL_MASK),
+        (Gdk.KEY_Page_Down, Gdk.ModifierType.CONTROL_MASK),
+        (Gdk.KEY_KP_Page_Down, Gdk.ModifierType.CONTROL_MASK),
+    )
+
     combobox_vcs = Gtk.Template.Child()
     console_vbox = Gtk.Template.Child()
     consoleview = Gtk.Template.Child()
@@ -183,6 +193,12 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         MeldDoc.__init__(self)
         bind_settings(self)
 
+        binding_set_names = ("GtkScrolledWindow", "GtkTreeView")
+        for set_name in binding_set_names:
+            binding_set = Gtk.binding_set_find(set_name)
+            for key, modifiers in self.replaced_entries:
+                Gtk.binding_entry_remove(binding_set, key, modifiers)
+
         # Set up per-view action group for top-level menu insertion
         self.view_action_group = Gio.SimpleActionGroup()
 
@@ -202,6 +218,7 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             ('previous-change', self.action_previous_change),
             ('refresh', self.action_refresh),
             ('vc-add', self.action_add),
+            ('vc-unstage', self.action_unstage),
             ('vc-commit', self.action_commit),
             ('vc-delete-locally', self.action_delete),
             ('vc-push', self.action_push),
@@ -217,6 +234,7 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
 
         new_boolean = GLib.Variant.new_boolean
         stateful_actions = (
+            ('vc-filter', None, GLib.Variant.new_boolean(False)),
             ('vc-flatten', self.action_filter_state_change,
                 new_boolean('flatten' in self.props.status_filters)),
             ('vc-status-modified', self.action_filter_state_change,
@@ -230,7 +248,8 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         )
         for (name, callback, state) in stateful_actions:
             action = Gio.SimpleAction.new_stateful(name, None, state)
-            action.connect('change-state', callback)
+            if callback:
+                action.connect('change-state', callback)
             self.view_action_group.add_action(action)
 
         builder = Gtk.Builder.new_from_resource(
@@ -240,8 +259,6 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         self.popup_menu.attach_to_widget(self)
 
         self.model = VcTreeStore()
-        self.connect("style-updated", self.model.on_style_updated)
-        self.model.on_style_updated(self)
         self.treeview.set_model(self.model)
         self.treeview.get_selection().connect(
             "changed", self.on_treeview_selection_changed)
@@ -377,7 +394,6 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         self.model.clear()
         self.filelabel.props.gfile = Gio.File.new_for_path(location)
         it = self.model.add_entries(None, [location])
-        self.treeview.grab_focus()
         self.treeview.get_selection().select_iter(it)
         self.model.set_path_state(it, 0, tree.STATE_NORMAL, isdir=1)
         self.recompute_label()
@@ -403,15 +419,28 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         self.scheduler.add_task(self.on_treeview_cursor_changed)
 
     def get_comparison(self):
-        uris = [Gio.File.new_for_path(self.location)]
+        if self.location:
+            uris = [Gio.File.new_for_path(self.location)]
+        else:
+            uris = []
         return RecentType.VersionControl, uris
 
     def recompute_label(self):
         self.label_text = os.path.basename(self.location)
-        # TRANSLATORS: This is the location of the directory being viewed
-        self.tooltip_text = _("Location: {path}").format(path=self.location)
+        self.tooltip_text = "\n".join((
+            # TRANSLATORS: This is the name of the version control
+            # system being used, e.g., "Git" or "Subversion"
+            _("{vc} comparison:").format(vc=self.vc.NAME),
+            self.location,
+        ))
         self.label_changed.emit(self.label_text, self.tooltip_text)
 
+    def set_labels(self, labels):
+        if labels:
+            self.filelabel.custom_label = labels[0]
+
+        self.recompute_label()
+
     def _search_recursively_iter(self, start_path, replace=False):
 
         # Initial yield so when we add this to our tasks, we don't
@@ -609,6 +638,7 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
         action_sensitivity = {
             'compare': 'compare' in valid_actions,
             'vc-add': 'add' in valid_actions,
+            'vc-unstage': 'unstage' in valid_actions,
             'vc-commit': 'commit' in valid_actions,
             'vc-delete-locally': bool(paths) and self.vc.root not in paths,
             'vc-push': 'push' in valid_actions,
@@ -723,6 +753,9 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
     def action_add(self, *args):
         self.vc.add(self.runner, self._get_selected_files())
 
+    def action_unstage(self, *args):
+        self.vc.unstage(self.runner, self._get_selected_files())
+
     def action_remove(self, *args):
         selected = self._get_selected_files()
         if any(os.path.isdir(p) for p in selected):
@@ -779,7 +812,8 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
             self.run_diff(f)
 
     def action_open_external(self, *args):
-        open_files_external(self._get_selected_files())
+        gfiles = [Gio.File.new_for_path(f) for f in self._get_selected_files() if f]
+        open_files_external(gfiles)
 
     def refresh(self):
         root = self.model.get_iter_first()
@@ -856,11 +890,11 @@ class VcView(Gtk.VBox, tree.TreeviewCommon, MeldDoc):
 
     @Gtk.Template.Callback()
     def on_treeview_popup_menu(self, treeview):
-        tree.TreeviewCommon.on_treeview_popup_menu(self, treeview)
+        return tree.TreeviewCommon.on_treeview_popup_menu(self, treeview)
 
     @Gtk.Template.Callback()
     def on_treeview_button_press_event(self, treeview, event):
-        tree.TreeviewCommon.on_treeview_button_press_event(
+        return tree.TreeviewCommon.on_treeview_button_press_event(
             self, treeview, event)
 
     @Gtk.Template.Callback()
